@@ -2,7 +2,6 @@
 
 from typing import List, Optional
 
-
 from llama_index.embeddings.base import BaseEmbedding
 
 # Google Universal Sentence Encode v5
@@ -28,6 +27,11 @@ class GoogleUnivSentEncoderEmbedding(BaseEmbedding):
         """Get query embedding."""
         return self._get_embedding(query)
 
+    # TODO: use proper async methods
+    async def _aget_text_embedding(self, query: str) -> List[float]:
+        """Get text embedding."""
+        return self._get_embedding(query)
+
     def _get_text_embedding(self, text: str) -> List[float]:
         """Get text embedding."""
         return self._get_embedding(text)
