@@ -3,8 +3,7 @@
 
 from typing import Any, List
 
-from langchain.embeddings.base import Embeddings as LCEmbeddings
-
+from llama_index.bridge.langchain import Embeddings as LCEmbeddings
 from llama_index.embeddings.base import BaseEmbedding
 
 
@@ -25,6 +24,13 @@ class LangchainEmbedding(BaseEmbedding):
         """Get query embedding."""
         return self._langchain_embedding.embed_query(query)
 
+    async def _aget_query_embedding(self, query: str) -> List[float]:
+        return await self._langchain_embedding.aembed_query(query)
+
+    async def _aget_text_embedding(self, text: str) -> List[float]:
+        embeds = await self._langchain_embedding.aembed_documents([text])
+        return embeds[0]
+
     def _get_text_embedding(self, text: str) -> List[float]:
         """Get text embedding."""
         return self._langchain_embedding.embed_documents([text])[0]
