@@ -1,16 +1,31 @@
 """NebulaGraph graph store index."""
 import logging
 import os
+from string import Template
 from typing import Any, Dict, List, Optional
 
+from tenacity import retry, stop_after_attempt, wait_random_exponential
+
 from llama_index.graph_stores.types import GraphStore
 
 QUOTE = '"'
 RETRY_TIMES = 3
+WAIT_MIN_SECONDS = 0.5
+WAIT_MAX_SECONDS = 10
 
 logger = logging.getLogger(__name__)
 
 
+rel_query = Template(
+    """
+MATCH ()-[e:`$edge_type`]->()
+  WITH e limit 1
+MATCH (m)-[:`$edge_type`]->(n) WHERE id(m) == src(e) AND id(n) == dst(e)
+RETURN "(:" + tags(m)[0] + ")-[:$edge_type]->(:" + tags(n)[0] + ")" AS rels
+"""
+)
+
+
 def hash_string_to_rank(string: str) -> int:
     # get signed 64-bit hash value
     signed_hash = hash(string)
@@ -47,6 +62,20 @@ def prepare_subjs_param(subjs: Optional[List[str]]) -> dict:
     return {"subjs": subjs_byte}
 
 
+def escape_str(value: str) -> str:
+    """Escape String for NebulaGraph Query."""
+    patterns = {
+        '"': " ",
+    }
+    for pattern in patterns:
+        if pattern in value:
+            value = value.replace(pattern, patterns[pattern])
+    if value[0] == " " or value[-1] == " ":
+        value = value.strip()
+
+    return value
+
+
 class NebulaGraphStore(GraphStore):
     """NebulaGraph graph store."""
 
@@ -54,8 +83,8 @@ class NebulaGraphStore(GraphStore):
         self,
         session_pool: Optional[Any] = None,
         space_name: Optional[str] = None,
-        edge_types: Optional[List[str]] = ["rel"],
-        rel_prop_names: Optional[List[str]] = ["predicate"],
+        edge_types: Optional[List[str]] = ["relationship"],
+        rel_prop_names: Optional[List[str]] = ["relationship"],
         tags: Optional[List[str]] = ["entity"],
         session_pool_kwargs: Optional[Dict[str, Any]] = {},
         **kwargs: Any,
@@ -80,13 +109,6 @@ class NebulaGraphStore(GraphStore):
         assert space_name is not None, "space_name should be provided."
         self._space_name = space_name
         self._session_pool_kwargs = session_pool_kwargs
-        if (
-            self._session_pool_kwargs is not None
-            and "retry" in self._session_pool_kwargs
-        ):
-            self._retry = self._session_pool_kwargs.pop("retry")
-        else:
-            self._retry = RETRY_TIMES
 
         if session_pool is None:
             self.init_session_pool()
@@ -141,6 +163,10 @@ class NebulaGraphStore(GraphStore):
         """Close NebulaGraph session pool."""
         self._session_pool.close()
 
+    @retry(
+        wait=wait_random_exponential(min=WAIT_MIN_SECONDS, max=WAIT_MAX_SECONDS),
+        stop=stop_after_attempt(RETRY_TIMES),
+    )
     def execute(self, query: str, param_map: Optional[Dict[str, Any]] = {}) -> Any:
         """Execute query.
 
@@ -154,30 +180,46 @@ class NebulaGraphStore(GraphStore):
         from nebula3.Exception import IOErrorException
         from nebula3.fbthrift.transport.TTransport import TTransportException
 
-        retry = self._retry
-        while retry > 0:
-            try:
-                result = self._session_pool.execute_parameter(query, param_map)
-                if not result.is_succeeded():
-                    raise ValueError(result.error_msg())
-                return result
-            except (TTransportException, IOErrorException) as e:
-                # connection issue, try to recreate session pool
-                if retry > 0:
-                    retry -= 2
-                    # try to recreate session pool
-                    self.init_session_pool()
-                else:
-                    raise e
-            except ValueError as e:
-                # query failed on db side
-                if retry > 0:
-                    retry -= 1
-                    continue
-                else:
-                    raise e
-            except Exception as e:
-                raise e
+        # Clean the query string by removing triple backticks
+        query = query.replace("```", "").strip()
+
+        try:
+            result = self._session_pool.execute_parameter(query, param_map)
+            if result is None:
+                raise ValueError(f"Query failed. Query: {query}, Param: {param_map}")
+            if not result.is_succeeded():
+                raise ValueError(
+                    f"Query failed. Query: {query}, Param: {param_map}"
+                    f"Error message: {result.error_msg()}"
+                )
+            return result
+        except (TTransportException, IOErrorException, RuntimeError) as e:
+            logger.error(
+                f"Connection issue, try to recreate session pool. Query: {query}, "
+                f"Param: {param_map}"
+                f"Erorr: {e}"
+            )
+            self.init_session_pool()
+            logger.info(
+                f"Session pool recreated. Query: {query}, Param: {param_map}"
+                f"This was due to error: {e}, and now retrying."
+            )
+            raise e
+
+        except ValueError as e:
+            # query failed on db side
+            logger.error(
+                f"Query failed. Query: {query}, Param: {param_map}"
+                f"Error message: {e}"
+            )
+            raise e
+        except Exception as e:
+            # other exceptions
+            logger.error(
+                f"Query failed. Query: {query}, Param: {param_map}"
+                f"Error message: {e}"
+            )
+            raise e
 
     @classmethod
     def from_dict(cls, config_dict: Dict[str, Any]) -> "GraphStore":
@@ -264,39 +306,56 @@ class NebulaGraphStore(GraphStore):
         # | "player100" | [1997, "team204"]                  |
         # ...
         # +-------------+------------------------------------+
+        rel_map: Dict[Any, List[Any]] = {}
+        if subjs is None or len(subjs) == 0:
+            # unlike simple graph_store, we don't do get_all here
+            return rel_map
+
         if len(self._edge_types) == 1:
-            # MATCH (s)-[e:follow*..2]->() WHERE id(s) IN ["player100", "player101"]
+            # MATCH (s)-[e:follow*..2]-() WHERE id(s) IN ["player100", "player101"]
             #   WITH id(s) AS subj, [rel in e | [rel.degree, dst(rel)] ] AS rels
+            #   WITH
+            #       subj,
+            #       REDUCE(acc = collect(NULL), l in rels | acc + l) AS flattened_rels
             # RETURN
             #   subj,
-            #   REDUCE(acc = collect(NULL), l in rels | acc + l) AS flattened_rels
+            #   REDUCE(acc = subj,l in flattened_rels|acc + ', ' + l) AS flattened_rels
             query = (
-                f"MATCH (s)-[e:`{self._edge_types[0]}`*..{depth}]->() "
+                f"MATCH (s)-[e:`{self._edge_types[0]}`*..{depth}]-() "
                 f"  WHERE id(s) IN $subjs "
                 f"WITH "
                 f"id(s) AS subj,"
                 f"[rel IN e | "
                 f"  [rel.`{self._rel_prop_names[0]}`, dst(rel)] "
                 f"] AS rels "
-                f"RETURN "
+                f"WITH "
                 f"  subj,"
                 f"  REDUCE(acc = collect(NULL), l in rels | acc + l)"
                 f"    AS flattened_rels"
+                f" RETURN"
+                f"  subj,"
+                f"  REDUCE(acc = subj, l in flattened_rels | acc + ', ' + l )"
+                f"    AS flattened_rels"
             )
         else:
             # edge_types = ["follow", "serve"]
             # rel_prop_names = ["degree", "start_year"]
-            # MATCH (s)-[e:follow|serve*..2]->()
+            # MATCH (s)-[e:follow|serve*..2]-()
             # WHERE id(s) IN ["player100", "player101"]
             #   WITH id(s) AS subj,
-            # [rel in e | [CASE type(rel)
+            #        [rel in e | [CASE type(rel)
             #     WHEN "follow" THEN rel.degree
             #     WHEN "serve" THEN rel.start_year
             #     END, dst(rel)] ]
             #     AS rels
+            #   WITH
+            #     subj,
+            #     REDUCE(acc = collect(NULL), l in rels | acc + l) AS
+            #       flattened_rels
             # RETURN
             #   subj,
-            #   REDUCE(acc = collect(NULL), l in rels | acc + l) AS flattened_rels
+            #   REDUCE(acc = subj, l in flattened_rels | acc + ', ' + l ) AS
+            #       flattened_rels
             _case_when_string = "".join(
                 [
                     f"WHEN {QUOTE}{edge_type}{QUOTE} THEN rel.`{rel_prop_name}` "
@@ -308,26 +367,32 @@ class NebulaGraphStore(GraphStore):
             query = (
                 f"MATCH (s)-[e:`{'`|`'.join(self._edge_types)}`*..{depth}]-() "
                 f"  WHERE id(s) IN $subjs "
-                f"WITH "
-                f"id(s) AS subj,"
-                f"[rel IN e | "
+                f"  WITH "
+                f"    id(s) AS subj,"
+                f" [rel IN e | "
                 f"  [CASE type(rel) "
                 f"  {_case_when_string}"
                 f"  END, dst(rel)] "
                 f"] AS rels "
+                f"  WITH"
+                f"    subj,"
+                f"    REDUCE(acc = collect(NULL), l in rels | acc + l) AS "
+                f"        flattened_rels"
                 f"RETURN"
                 f"  subj,"
-                f"  REDUCE(acc = collect(NULL), l in rels | acc + l) AS flattened_rels"
+                f"  REDUCE(acc = subj, l in flattened_rels | acc + ', ' + l ) AS "
+                f"      flattened_rels"
             )
         subjs_param = prepare_subjs_param(subjs)
         logger.debug(f"get_flat_rel_map() subjs_param: {subjs}, query: {query}")
         result = self.execute(query, subjs_param)
+        if result is None:
+            return rel_map
 
         # get raw data
         subjs_ = result.column_values("subj") or []
         rels_ = result.column_values("flattened_rels") or []
 
-        rel_map: Dict[Any, List[Any]] = {}
         for subj, rel in zip(subjs_, rels_):
             subj_ = subj.cast()
             rel_ = rel.cast()
@@ -344,8 +409,10 @@ class NebulaGraphStore(GraphStore):
         # SimpleGraphStore.get_rel_map() though.
         # But this makes more sense for multi-hop relation path.
 
-        # lower case subjs
-        subjs = [subj.lower() for subj in subjs] if subjs else None
+        if subjs is not None:
+            subjs = [escape_str(subj) for subj in subjs]
+            if len(subjs) == 0:
+                return {}
 
         return self.get_flat_rel_map(subjs, depth)
 
@@ -360,19 +427,19 @@ class NebulaGraphStore(GraphStore):
         # thus we have to assume subj to be the first entity.tag_name
 
         # lower case subj, rel, obj
-        subj = subj.lower()
-        rel = rel.lower()
-        obj = obj.lower()
+        subj = escape_str(subj)
+        rel = escape_str(rel)
+        obj = escape_str(obj)
 
         edge_type = self._edge_types[0]
         rel_prop_name = self._rel_prop_names[0]
         entity_type = self._tags[0]
         rel_hash = hash_string_to_rank(rel)
         dml_query = (
-            f"INSERT VERTEX `{entity_type}`() "
-            f"  VALUES {QUOTE}{subj}{QUOTE}:();"
-            f"INSERT VERTEX `{entity_type}`() "
-            f"  VALUES {QUOTE}{obj}{QUOTE}:();"
+            f"INSERT VERTEX `{entity_type}`(name) "
+            f"  VALUES {QUOTE}{subj}{QUOTE}:({QUOTE}{subj}{QUOTE});"
+            f"INSERT VERTEX `{entity_type}`(name) "
+            f"  VALUES {QUOTE}{obj}{QUOTE}:({QUOTE}{obj}{QUOTE});"
             f"INSERT EDGE `{edge_type}`(`{rel_prop_name}`) "
             f"  VALUES "
             f"{QUOTE}{subj}{QUOTE}->{QUOTE}{obj}{QUOTE}"
@@ -381,7 +448,7 @@ class NebulaGraphStore(GraphStore):
         logger.debug(f"upsert_triplet() DML query: {dml_query}")
         result = self.execute(dml_query)
         assert (
-            result.is_succeeded()
+            result and result.is_succeeded()
         ), f"Failed to upsert triplet: {subj} {rel} {obj}, query: {dml_query}"
 
     def delete(self, subj: str, rel: str, obj: str) -> None:
@@ -394,9 +461,9 @@ class NebulaGraphStore(GraphStore):
         """
 
         # lower case subj, rel, obj
-        subj = subj.lower()
-        rel = rel.lower()
-        obj = obj.lower()
+        subj = escape_str(subj)
+        rel = escape_str(rel)
+        obj = escape_str(obj)
 
         # DELETE EDGE serve "player100" -> "team204"@7696463696635583936;
         edge_type = self._edge_types[0]
@@ -409,7 +476,7 @@ class NebulaGraphStore(GraphStore):
         logger.debug(f"delete() DML query: {dml_query}")
         result = self.execute(dml_query)
         assert (
-            result.is_succeeded()
+            result and result.is_succeeded()
         ), f"Failed to delete triplet: {subj} {rel} {obj}, query: {dml_query}"
         # Get isolated vertices to be deleted
         # MATCH (s) WHERE id(s) IN ["player700"] AND NOT (s)-[]-()
@@ -430,5 +497,58 @@ class NebulaGraphStore(GraphStore):
 
         result = self.execute(dml_query)
         assert (
-            result.is_succeeded()
+            result and result.is_succeeded()
         ), f"Failed to delete isolated vertices: {isolated}, query: {dml_query}"
+
+    def refresh_schema(self) -> None:
+        """
+        Refreshes the NebulaGraph Store Schema.
+        """
+        tags_schema, edge_types_schema, relationships = [], [], []
+        for tag in self.execute("SHOW TAGS").column_values("Name"):
+            tag_name = tag.cast()
+            tag_schema = {"tag": tag_name, "properties": []}
+            r = self.execute(f"DESCRIBE TAG `{tag_name}`")
+            props, types = r.column_values("Field"), r.column_values("Type")
+            for i in range(r.row_size()):
+                tag_schema["properties"].append((props[i].cast(), types[i].cast()))
+            tags_schema.append(tag_schema)
+        for edge_type in self.execute("SHOW EDGES").column_values("Name"):
+            edge_type_name = edge_type.cast()
+            edge_schema = {"edge": edge_type_name, "properties": []}
+            r = self.execute(f"DESCRIBE EDGE `{edge_type_name}`")
+            props, types = r.column_values("Field"), r.column_values("Type")
+            for i in range(r.row_size()):
+                edge_schema["properties"].append((props[i].cast(), types[i].cast()))
+            edge_types_schema.append(edge_schema)
+
+            # build relationships types
+            r = self.execute(
+                rel_query.substitute(edge_type=edge_type_name)
+            ).column_values("rels")
+            if len(r) > 0:
+                relationships.append(r[0].cast())
+
+        self.schema = (
+            f"Node properties: {tags_schema}\n"
+            f"Edge properties: {edge_types_schema}\n"
+            f"Relationships: {relationships}\n"
+        )
+
+    def get_schema(self, refresh: bool = False) -> str:
+        """Get the schema of the NebulaGraph store."""
+        if self.schema and not refresh:
+            return self.schema
+        self.refresh_schema()
+        logger.debug(f"get_schema() schema:\n{self.schema}")
+        return self.schema
+
+    def query(self, query: str, param_map: Optional[Dict[str, Any]] = {}) -> Any:
+        result = self.execute(query, param_map)
+        columns = result.keys()
+        d: Dict[str, list] = {}
+        for col_num in range(result.col_size()):
+            col_name = columns[col_num]
+            col_list = result.column_values(col_name)
+            d[col_name] = [x.cast() for x in col_list]
+        return d
