@@ -1,7 +1,9 @@
 from enum import Enum
 from typing import Dict, Type
 
+from llama_index.graph_stores.kuzu import KuzuGraphStore
 from llama_index.graph_stores.nebulagraph import NebulaGraphStore
+from llama_index.graph_stores.neo4j import Neo4jGraphStore
 from llama_index.graph_stores.simple import SimpleGraphStore
 from llama_index.graph_stores.types import GraphStore
 
@@ -9,11 +11,15 @@ from llama_index.graph_stores.types import GraphStore
 class GraphStoreType(str, Enum):
     SIMPLE = "simple_kg"
     NEBULA = "nebulagraph"
+    KUZU = "kuzu"
+    NEO4J = "neo4j"
 
 
 GRAPH_STORE_TYPE_TO_GRAPH_STORE_CLASS: Dict[GraphStoreType, Type[GraphStore]] = {
     GraphStoreType.SIMPLE: SimpleGraphStore,
     GraphStoreType.NEBULA: NebulaGraphStore,
+    GraphStoreType.KUZU: KuzuGraphStore,
+    GraphStoreType.NEO4J: Neo4jGraphStore,
 }
 
 GRAPH_STORE_CLASS_TO_GRAPH_STORE_TYPE: Dict[Type[GraphStore], GraphStoreType] = {
