@@ -1,12 +1,12 @@
 """Default query for EmptyIndex."""
 from typing import Any, List, Optional
 
-from llama_index.data_structs.node import NodeWithScore
 from llama_index.indices.base_retriever import BaseRetriever
 from llama_index.indices.empty.base import EmptyIndex
 from llama_index.indices.query.schema import QueryBundle
 from llama_index.prompts.default_prompts import DEFAULT_SIMPLE_INPUT_PROMPT
 from llama_index.prompts.prompts import SimpleInputPrompt
+from llama_index.schema import NodeWithScore
 
 
 class EmptyIndexRetriever(BaseRetriever):
