@@ -1,8 +1,8 @@
 from abc import ABC, abstractmethod
 from typing import List, Optional
 
-from llama_index.data_structs.node import NodeWithScore
 from llama_index.indices.query.schema import QueryBundle
+from llama_index.schema import NodeWithScore
 
 
 class BaseNodePostprocessor(ABC):
