@@ -1,9 +1,7 @@
 from abc import ABC, abstractmethod
-from typing import Any, Generic, Type, TypeVar
+from typing import Any, Generic, Type
 
-from pydantic import BaseModel
-
-Model = TypeVar("Model", bound=BaseModel)
+from llama_index.types import Model
 
 
 class BasePydanticProgram(ABC, Generic[Model]):
@@ -20,3 +18,6 @@ class BasePydanticProgram(ABC, Generic[Model]):
     @abstractmethod
     def __call__(self, *args: Any, **kwds: Any) -> Model:
         pass
+
+    async def acall(self, *args: Any, **kwds: Any) -> Model:
+        return self(*args, **kwds)
