@@ -1,5 +1,11 @@
+from llama_index.query_engine.citation_query_engine import CitationQueryEngine
+from llama_index.query_engine.flare.base import FLAREInstructQueryEngine
 from llama_index.query_engine.graph_query_engine import ComposableGraphQueryEngine
+from llama_index.query_engine.knowledge_graph_query_engine import (
+    KnowledgeGraphQueryEngine,
+)
 from llama_index.query_engine.multistep_query_engine import MultiStepQueryEngine
+from llama_index.query_engine.pandas_query_engine import PandasQueryEngine
 from llama_index.query_engine.retriever_query_engine import RetrieverQueryEngine
 from llama_index.query_engine.retry_query_engine import (
     RetryGuidelineQueryEngine,
@@ -9,12 +15,15 @@ from llama_index.query_engine.retry_source_query_engine import RetrySourceQueryE
 from llama_index.query_engine.router_query_engine import (
     RetrieverRouterQueryEngine,
     RouterQueryEngine,
+    ToolRetrieverRouterQueryEngine,
 )
-from llama_index.query_engine.sql_vector_query_engine import SQLAutoVectorQueryEngine
 from llama_index.query_engine.sql_join_query_engine import SQLJoinQueryEngine
-from llama_index.query_engine.sub_question_query_engine import SubQuestionQueryEngine
+from llama_index.query_engine.sql_vector_query_engine import SQLAutoVectorQueryEngine
+from llama_index.query_engine.sub_question_query_engine import (
+    SubQuestionQueryEngine,
+    SubQuestionAnswerPair,
+)
 from llama_index.query_engine.transform_query_engine import TransformQueryEngine
-from llama_index.query_engine.flare.base import FLAREInstructQueryEngine
 
 __all__ = [
     "CitationQueryEngine",
@@ -24,11 +33,15 @@ __all__ = [
     "MultiStepQueryEngine",
     "RouterQueryEngine",
     "RetrieverRouterQueryEngine",
+    "ToolRetrieverRouterQueryEngine",
     "SubQuestionQueryEngine",
+    "SubQuestionAnswerPair",
     "SQLJoinQueryEngine",
     "SQLAutoVectorQueryEngine",
     "RetryQueryEngine",
     "RetrySourceQueryEngine",
     "RetryGuidelineQueryEngine",
     "FLAREInstructQueryEngine",
+    "PandasQueryEngine",
+    "KnowledgeGraphQueryEngine",
 ]
