@@ -2,7 +2,7 @@
 from typing import List
 
 from llama_index.readers.base import BaseReader
-from llama_index.readers.schema.base import Document
+from llama_index.schema import Document
 
 
 class StringIterableReader(BaseReader):
@@ -28,6 +28,6 @@ class StringIterableReader(BaseReader):
         """Load the data."""
         results = []
         for text in texts:
-            results.append(Document(text))
+            results.append(Document(text=text))
 
         return results
