@@ -17,6 +17,8 @@ from llama_index.indices.vector_store.retrievers import (
     VectorIndexRetriever,
 )
 from llama_index.retrievers.transform_retriever import TransformRetriever
+from llama_index.retrievers.recursive_retriever import RecursiveRetriever
+from llama_index.retrievers.router_retriever import RouterRetriever
 
 __all__ = [
     "VectorIndexRetriever",
@@ -32,4 +34,6 @@ __all__ = [
     "TransformRetriever",
     "KeywordTableSimpleRetriever",
     "BaseRetriever",
+    "RecursiveRetriever",
+    "RouterRetriever",
 ]
