@@ -1,5 +1,6 @@
 from llama_index.storage.index_store.keyval_index_store import KVIndexStore
 from llama_index.storage.index_store.simple_index_store import SimpleIndexStore
 from llama_index.storage.index_store.mongo_index_store import MongoIndexStore
+from llama_index.storage.index_store.redis_index_store import RedisIndexStore
 
-__all__ = ["KVIndexStore", "SimpleIndexStore", "MongoIndexStore"]
+__all__ = ["KVIndexStore", "SimpleIndexStore", "MongoIndexStore", "RedisIndexStore"]
