@@ -1,4 +1,5 @@
 from llama_index.storage.kvstore.simple_kvstore import SimpleKVStore
 from llama_index.storage.kvstore.mongodb_kvstore import MongoDBKVStore
+from llama_index.storage.kvstore.redis_kvstore import RedisKVStore
 
-__all__ = ["SimpleKVStore", "MongoDBKVStore"]
+__all__ = ["SimpleKVStore", "MongoDBKVStore", "RedisKVStore"]
