@@ -20,10 +20,19 @@ class MockEmbedding(BaseEmbedding):
         super().__init__(*args, **kwargs)
         self.embed_dim = embed_dim
 
+    def _get_vector(self) -> List[float]:
+        return [0.5] * self.embed_dim
+
+    async def _aget_text_embedding(self, text: str) -> List[float]:
+        return self._get_vector()
+
+    async def _aget_query_embedding(self, query: str) -> List[float]:
+        return self._get_vector()
+
     def _get_query_embedding(self, query: str) -> List[float]:
         """Get query embedding."""
-        return [0.5] * self.embed_dim
+        return self._get_vector()
 
     def _get_text_embedding(self, text: str) -> List[float]:
         """Get text embedding."""
-        return [0.5] * self.embed_dim
+        return self._get_vector()
