@@ -3,7 +3,7 @@ from unittest.mock import patch
 from llama_index.indices.keyword_table.simple_base import SimpleKeywordTableIndex
 from llama_index.indices.query.schema import QueryBundle
 from llama_index.indices.service_context import ServiceContext
-from llama_index.readers.schema.base import Document
+from llama_index.schema import Document
 from tests.mock_utils.mock_utils import mock_extract_keywords
 
 
@@ -29,4 +29,4 @@ def test_retrieve(
     retriever = table.as_retriever(retriever_mode="simple")
     nodes = retriever.retrieve(QueryBundle("Hello"))
     assert len(nodes) == 1
-    assert nodes[0].node.text == "Hello world."
+    assert nodes[0].node.get_content() == "Hello world."
