@@ -3,7 +3,7 @@
 from typing import List
 import pytest
 
-from llama_index.readers.schema.base import Document
+from llama_index.schema import Document
 
 
 @pytest.fixture
@@ -16,4 +16,4 @@ def documents() -> List[Document]:
         "This is another test.\n"
         "This is a test v2."
     )
-    return [Document(doc_text)]
+    return [Document(text=doc_text)]
