@@ -1,6 +1,7 @@
 import sys
 from unittest.mock import MagicMock
-from llama_index.data_structs.node import DocumentRelationship, Node
+
+from llama_index.schema import NodeRelationship, RelatedNodeInfo, TextNode
 from llama_index.vector_stores.types import NodeWithEmbedding
 
 from llama_index.vector_stores.weaviate import WeaviateVectorStore
@@ -18,10 +19,12 @@ def test_weaviate_add() -> None:
     vector_store.add(
         [
             NodeWithEmbedding(
-                node=Node(
+                node=TextNode(
                     text="test node text",
-                    doc_id="test node id",
-                    relationships={DocumentRelationship.SOURCE: "test doc id"},
+                    id_="test node id",
+                    relationships={
+                        NodeRelationship.SOURCE: RelatedNodeInfo(node_id="test doc id")
+                    },
                 ),
                 embedding=[0.5, 0.5],
             )
